import { NextPage } from 'next';
import Title from 'components/Title';
import Container from 'components/Container';
import axios from 'axios';
import { useEffect, useState } from 'react';
import { formatBytes } from 'utils/formatBytes';
import { K8sDataType } from 'types/types';
import Head from 'next/head';
import Header from 'layout/Header';
import Link from 'next/link';

interface HomeProps {
  environment: string;
}

const Home: NextPage<HomeProps> = ({
  environment
}) => {

  const [
    nodeData2,
    setNodeData2
  ] = useState<K8sDataType>();

  const [
    nodeData1,
    setNodeData1
  ] = useState<K8sDataType>();

  const fetchData = async(node: '01' | '02') => {
    const { data } = await axios.get(`/api/data?node=${node}`);
    if (node === '01') {
      setNodeData1(data);
    } else if (node === '02') {
      setNodeData2(data);
    }
  }

  useEffect(() => {
    fetchData('01');
    fetchData('02');
    const interval = setInterval(async() => {
      await fetchData('01');
      await fetchData('02');
    }, 5000);
    return () => {
      clearInterval(interval);
    }
  }, [])

  const cluserNodeLabels = (nodeData: K8sDataType) => [
    {
      title: 'RAM in use',
      value: nodeData?.node?.memory?.workingSetBytes ? formatBytes(nodeData?.node?.memory?.workingSetBytes) : 'Loading...'
    },
    {
      title: 'RAM available',
      value: nodeData?.node?.memory?.availableBytes ? formatBytes(nodeData?.node?.memory?.availableBytes) : 'Loading...'
    },
    {
      title: 'Disk space',
      value: nodeData?.node?.fs?.availableBytes && nodeData?.node?.fs?.capacityBytes ?
        `${formatBytes(nodeData?.node?.fs?.availableBytes)} / ${formatBytes(nodeData?.node?.fs?.capacityBytes)}` : 'Loading...'
    },
    {
      title: 'CPU usage',
      value: nodeData?.node?.cpu?.usageCoreNanoSeconds && nodeData?.node?.cpu?.usageNanoCores ?
        `${((nodeData?.node?.cpu?.usageCoreNanoSeconds / (nodeData?.node?.cpu?.usageNanoCores * 1e9)) * 100).toFixed(1)} %` : 'Loading...'
    }
  ]

  const renderLabel = (value: string, title: string, idx: number) => {
    return (
      <div className='bg-white text-center border-[3px] py-2 px-5 shadow-xl border-primary rounded-2xl h-full flex items-center flex-col justify-center' key={idx}>
        <span className='text-[36px] font-bold'>{`${value}`}</span>
        <span className='block text-[25px]'>{title}</span>
      </div>
    )
  }

  return (
    <>
      <Head>
        <title>Next.js - FIM UHK</title>
      </Head>
      <Header environment={environment}>
        <Title className='font-bold max-w-5xl mx-auto' as='h1' content='Welcome to Next.js application which runs in Kubernetes cluster' />
        <h2 className='text-h4 font-bold italic'>for University Qualification Thesis purpose</h2>
        <Link href={'/about'}>
          <a><h2 className='text-h4 font-bold mt-5 py-2 px-5 bg-primary text-white inline-block rounded-2xl transition-all hover:scale-105'>Read more about this project</h2></a>
        </Link>
      </Header>
      <Container width='1120'>
        <Title as='h2' content='Cluster' />
        <Title as='h3' content={'Node 1'} />
        <div className='grid grid-cols-1 lg:grid-cols-4 gap-4 items-center'>
          {
            nodeData1 &&
            cluserNodeLabels(nodeData1)?.map(({ title, value }, idx) => {
              return renderLabel(value, title, idx);
            })
          }
        </div>
        <Title className='mt-16' as='h3' content={'Node 2'} />
        <div className='grid grid-cols-1 lg:grid-cols-4 gap-4 items-center'>
          {
            nodeData2 &&
            cluserNodeLabels(nodeData2)?.map(({ title, value }, idx) => {
              return renderLabel(value, title, idx);
            })
          }
        </div>
      </Container>
    </>
  )
}

export const getServerSideProps = async({ locale }: { locale: any }) => {
  return {
    props: {
      environment: process.env.ENVIRONMENT
    },
  };
}

export default Home;